@@ -23,40 +23,76 @@
 # esc 1 menu 62 home 63 fullscreen 64 plus 65 minus 66
 # shift 42 ctrl 29 alt 100 back 14
 
-# do not run bootmenu when device is not un USER state
-if [ "`getbootstate`" != "USER" ]; then
+# do not run bootmenu when device is not in USER state
+# /sbin/preinit on N900 storing state to variable $STATE
+if [ -n "$STATE" -a "$STATE" != "USER" ]; then
 	return
 fi
 
-# create needed devices to boot
-if [ ! -c /dev/null ]; then
-	rm -f /dev/null
-	mknod /dev/null c 1 3
-	chown root:root /dev/null
-	chmod 666 /dev/null
+# product in /proc/component_version
+#  SU-18 - Nokia 770
+#  RX-34 - Nokia N800
+#  RX-44 - Nokia N810
+#  RX-48 - Nokia N810 WiMax
+#  RX-51 - Nokia N900
+[ -f /proc/component_version ] && PRODUCT=`awk '/product/ { print $2 }' /proc/component_version`
+
+# Hardware in /proc/cpuinfo
+#  Nokia 770
+#  Nokia N800
+#  Nokia N810
+#  Nokia N810 WiMAX
+#  Nokia RX-51 board
+if [ -z "$PRODUCT" ] && [ -f /proc/cpuinfo ]; then
+	HARDWARE=`sed -n 's/Hardware\s*:\s*//p' /proc/cpuinfo`
+	case "$HARDWARE" in
+	"Nokia 770") PRODUCT="SU-18";;
+	"Nokia N800") PRODUCT="RX-34";;
+	"Nokia N810") PRODUCT="RX-48";;
+	"Nokia N810 WiMAX") PRODUCT="RX-48";;
+	"Nokia RX-51 board") PRODUCT="RX-51";;
+	esac
 fi
-if [ ! -c /dev/urandom ]; then
-	mknod /dev/urandom c 1 9
-	chown root:root /dev/urandom
-	chmod 444 /dev/urandom
-fi
-if [ ! -c /dev/console ]; then
-	rm -f /dev/console
-	mknod /dev/console c 5 1
-	chown root:tty /dev/console
-	chmod 600 /dev/console
-fi
-if [ ! -c /dev/fb0 ]; then
-	rm -f /dev/fb0
-	mknod /dev/fb0 c 29 0
-	chown root:video /dev/fb0
-	chmod 660 /dev/fb0
-fi
-if [ ! -c /dev/mtd1 ]; then
-	rm -f /dev/mtd1
-	mknod /dev/mtd1 c 90 2
-	chown root:root /dev/mtd1
-	chmod 600 /dev/mtd1
+
+# create needed devices to boot on RX-51
+# new /sbin/preinit doing it too, check for devfs_created
+if [ "$PRODUCT" = "RX-51" ] && [ "$devfs_created" != "1" ]; then
+	if [ ! -c /dev/null ]; then
+		rm -f /dev/null
+		mknod /dev/null c 1 3
+		chown root:root /dev/null
+		chmod 666 /dev/null
+	fi
+	if [ ! -c /dev/urandom ]; then
+		rm -f /dev/urandom
+		mknod /dev/urandom c 1 9
+		chown root:root /dev/urandom
+		chmod 444 /dev/urandom
+	fi
+	if [ ! -c /dev/console ]; then
+		rm -f /dev/console
+		mknod /dev/console c 5 1
+		chown root:tty /dev/console
+		chmod 600 /dev/console
+	fi
+	if [ ! -c /dev/fb0 ]; then
+		rm -f /dev/fb0
+		mknod /dev/fb0 c 29 0
+		chown root:video /dev/fb0
+		chmod 660 /dev/fb0
+	fi
+	if [ ! -c /dev/mtd1 ]; then
+		rm -f /dev/mtd1
+		mknod /dev/mtd1 c 90 2
+		chown root:root /dev/mtd1
+		chmod 600 /dev/mtd1
+	fi
+	if [ ! -c /dev/tty1 ]; then
+		rm -f /dev/tty1
+		mknod /dev/tty1 c 4 1
+		chown root:tty /dev/tty1
+		chmod 600 /dev/tty1
+	fi
 fi
 
 # program for writing text on screen
@@ -66,14 +102,6 @@ echo "Loading BOOT MENU..."
 $T2S -x 200 -y 0 -w 400 -h 55 -c
 $T2S -s 2 -H center -y 20 -T 0 -t "Loading BOOT MENU..."
 
-# Nokia Product:
-#  SU-18 - Nokia 770
-#  RX-34 - Nokia N800
-#  RX-44 - Nokia N810
-#  RX-48 - Nokia N810 WiMax
-#  RX-51 - Nokia N900
-PRODUCT=`awk '/product/ { print $2 }' /proc/component_version`
-
 # keyboard event device
 case $PRODUCT in
 	SU-18|RX-51) EVNAME=event1;;
@@ -93,6 +121,14 @@ case $PRODUCT in
 	# TODO: Add brightness devices for other products
 esac
 
+if [ "$PRODUCT" = "RX-51" ]; then
+	for p in /sys/class/input/event*; do
+		n=`basename $p`
+		grep -q 'twl4030_keypad' "/sys/class/input/$n/device/phys" && EVNAME=$n
+		grep -q 'twl4030_pwrbutton' "/sys/class/input/$n/device/phys" && EVPOWER=$n
+	done
+fi
+
 # set display brightness
 [ "$BRIGHTNESS" != "" ] && echo 100 > $BRIGHTNESS
 
@@ -101,12 +137,10 @@ if [ "$PRODUCT" = "RX-51" ]; then
 
 	(
 	# Led
-	insmod $MODULE_PATH/led-class.ko
-	insmod $MODULE_PATH/leds-lp5523.ko
+	modprobe leds-lp5523
 
 	# RTC
-	insmod $MODULE_PATH/rtc-core.ko
-	insmod $MODULE_PATH/rtc-twl4030.ko
+	modprobe rtc-twl4030 || modprobe rtc-twl
 	sleep 2
 	for p in /sys/class/rtc/rtc*/; do
 		n=`basename $p | sed "s/.*://"`
@@ -115,22 +149,28 @@ if [ "$PRODUCT" = "RX-51" ]; then
 	done
 
 	# Watchdogs
-	insmod $MODULE_PATH/omap_wdt.ko
-	insmod $MODULE_PATH/twl4030_wdt.ko
+	modprobe omap_wdt
+	modprobe twl4030_wdt
+	rm -f /dev/twl4030_wdt /dev/watchdog*
 	sleep 2
 	for p in /sys/bus/platform/devices/*_wdt/misc:*; do
+		if [ ! -e "$p" ]; then continue; fi
 		n=`basename $p | sed "s/.*://"`
 		rm -f /dev/$n
 		mknod /dev/$n c `cat $p/dev | sed "s/:/ /g"`
 	done
+	for p in /sys/class/watchdog/watchdog*; do
+		if [ ! -e "$p" ]; then continue; fi
+		n=`basename $p`
+		rm -f /dev/$n
+		mknod /dev/$n c `cat $p/dev | sed "s/:/ /g"`
+	done
 	/usr/sbin/watchdogs
 	) &
 
 	# eMMC and SD
 	cd /dev; /sbin/MAKEDEV input; cd /
-	insmod $MODULE_PATH/mmc_core.ko
-	insmod $MODULE_PATH/mmc_block.ko
-	insmod $MODULE_PATH/omap_hsmmc.ko
+	modprobe omap_hsmmc
 	rm -f /dev/mmcblk*
 	sleep 2
 	for p in /sys/block/mmcblk*/ /sys/block/mmcblk*/mmcblk*p*/; do
@@ -139,6 +179,9 @@ if [ "$PRODUCT" = "RX-51" ]; then
 		mknod /dev/$n b `cat $p/dev | sed "s/:/ /g"`
 	done
 
+	# Always show bootmenu
+	show_bootmenu=yes
+
 fi
 
 # wait 1 second for keyup event
@@ -164,6 +207,7 @@ KEY_BACK=14
 
 # IP address for USB networking (same as in /etc/network/interfaces)
 USB_IP=192.168.2.15
+USB_GW=192.168.2.14
 
 # default menu timeout
 MENU_TIMEOUT=30
@@ -235,9 +279,7 @@ MENU_IDX=$MENU_MIN
 load_modules()
 {
 	for mod in $*; do
-		if [ -f $MODULE_PATH/${mod}.ko ]; then
-			insmod $MODULE_PATH/${mod}.ko
-		fi
+		modprobe $mod
 	done
 }
 
@@ -290,18 +332,26 @@ set_usb_mode()
 usb_mass_storage()
 {
 	${T2S} -c
-	DEVICES=
-	[ "$INT_CARD" != "" ] && DEVICES=$DEVICES,/dev/$INT_CARD
-	[ "$EXT_CARD" != "" ] && DEVICES=$DEVICES,/dev/$EXT_CARD
+	if [ "$INT_CARD" != "" ] && [ "$EXT_CARD" != "" ]; then
+		DEVICES=/dev/$INT_CARD,/dev/$EXT_CARD
+	elif [ "$INT_CARD" != "" ]; then
+		DEVICES=/dev/$INT_CARD
+	elif [ "$EXT_CARD" != "" ]; then
+		DEVICES=/dev/$EXT_CARD
+	else
+		DEVICES=
+	fi
 
 	for MODULE in g_nokia g_ether phonet; do
 		if lsmod | grep -q $MODULE; then rmmod $MODULE; fi
 	done
 
-	insmod $MODULE_PATH/g_file_storage.ko file=$DEVICES stall=0 removable=1
+	modprobe g_file_storage file=$DEVICES stall=0 removable || modprobe g_mass_storage file=$DEVICES stall=0 removable=1,1
 	y=20
 	${T2S} -s 2 -H center -y $y -T 0 -t "USB mass storage mode started"
 
+	y=$((y+20)); ${T2S} -s 2 -H center -y $y -T 0 -t "Exported devices: $DEVICES"
+
 	y=$((y+40)); ${T2S} -s 2 -H center -y $y -T 0 -t "Press ESC/SHIFT key to disconnect and go back"
 
 	while true; do
@@ -311,6 +361,7 @@ usb_mass_storage()
 	done
 
 	rmmod g_file_storage
+	rmmod g_mass_storage
 }
 
 usbnet()
@@ -318,13 +369,13 @@ usbnet()
 	${T2S} -c
 	[ -c /dev/ptmx ] || mknod /dev/ptmx c 5 2
 	[ -d /dev/pts ] || mkdir /dev/pts
+	mount -t devpts devpts /dev/pts
 
 	if [ "$PRODUCT" = "RX-51" ]; then
-		insmod $MODULE_PATH/phonet.ko
-		insmod $MODULE_PATH/g_nokia.ko
+		modprobe g_nokia
 		hwclock -s # load system clock
 	else
-		insmod $MODULE_PATH/g_ether.ko
+		modprobe g_ether
 		retutime -i # load system clock
 	fi
 
@@ -332,39 +383,22 @@ usbnet()
 		hostname -F /etc/hostname
 	fi
 
-	cat > /var/run/resolv.conf.usb0 << EOF
-nameserver ${USB_IP%.*}.$((${USB_IP##*.}-1))
-nameserver 8.8.8.8
-EOF
-
-	grep -q "/var/run/resolv.conf.usb0" /etc/dnsmasq.conf || echo "resolv-file=/var/run/resolv.conf.usb0" >> /etc/dnsmasq.conf
-
 	ifconfig usb0 $USB_IP up
 	ifconfig lo 127.0.0.1 up
-	route add default gw ${USB_IP%.*}.$((${USB_IP##*.}-1))
-	/usr/sbin/dnsmasq -i lo -a 127.0.0.1 -z
-
-	mount -t devpts devpts /dev/pts
+	route add default gw $USB_GW
 
 	y=20
-	${T2S} -s 2 -H center -y $y -T 0 -t "USB networking is up with IP $USB_IP"
+	${T2S} -s 2 -H center -y $y -T 0 -t "USB networking is up"
 
-	if [ -x /usr/sbin/udhcpd ]; then
-		if [ -f /var/run/udhcpd.pid ]; then kill `cat /var/run/udhcpd.pid`; fi
-		cat > /var/run/udhcpd.conf << EOF
-pidfile /var/run/udhcpd.pid
-interface usb0
-static_lease 00:00:00:00:00:00 usb0
-lease_file /dev/null
-start ${USB_IP%.*}.$((${USB_IP##*.}-1))
-end ${USB_IP%.*}.$((${USB_IP##*.}-1))
-option subnet usb0
-EOF
-		/usr/sbin/udhcpd /var/run/udhcpd.conf &
-		y=$((y+20)); ${T2S} -s 2 -H center -y $y -T 0 -t "dhcp server started"
-	else
-		y=$((y+20)); ${T2S} -s 2 -H center -y $y -T 0 -t "dhcp server not found"
-	fi
+	y=$((y+20)); ${T2S} -s 2 -H center -y $y -T 0 -t "local: $USB_IP"
+	y=$((y+20)); ${T2S} -s 2 -H center -y $y -T 0 -t "remote: $USB_GW"
+
+	echo "nameserver $USB_GW" > /var/run/resolv.conf.usb0
+	grep -q "/var/run/resolv.conf.usb0" /etc/dnsmasq.conf || echo "resolv-file=/var/run/resolv.conf.usb0" >> /etc/dnsmasq.conf
+	/usr/sbin/dnsmasq -i lo -a 127.0.0.1 -z -x /var/run/dnsmasq.pid.lo
+
+	/usr/sbin/dnsmasq -i usb0 -I lo -a $USB_IP -z -x /var/run/dnsmasq.pid.usb0 -C /dev/null -F $USB_GW,$USB_GW -K -9 -l /dev/null -p 0 -O 3 -O 6
+	y=$((y+20)); ${T2S} -s 2 -H center -y $y -T 0 -t "dhcp server dnsmasq started on port 67"
 
 	if [ -x /usr/sbin/utelnetd ]; then # start utelnetd
 		/usr/sbin/utelnetd -l /bin/sh -d
@@ -420,10 +454,9 @@ EOF
 	killall sshd
 	killall dropbear
 	killall utelnetd
-	killall udhcpd
 	killall dnsmasq
-	kill `cat /var/run/udhcpd.pid`
-	rm -f /var/run/udhcpd.pid /var/run/udhcpd.conf
+	rm -f /var/run/dnsmasq.pid.lo
+	rm -f /var/run/dnsmasq.pid.usb0
 	rm -f /var/run/resolv.conf.usb0
 	rm -f /var/run/avahi-daemon.conf
 	route del default
@@ -524,20 +557,16 @@ menu_loop()
 	done
 }
 
-if [ "$PRODUCT" != "RX-51" ]; then
-	# handle ask:xxx syntax
-	case "$default_root" in
-		ask)
-			show_bootmenu=yes
-			;;
-		ask:*)
-			default_root=$(expr substr $default_root 5 100)
-			show_bootmenu=yes
-			;;
-	esac
-else
-	show_bootmenu=yes
-fi
+# handle ask:xxx syntax
+case "$default_root" in
+	ask|"")
+		show_bootmenu=yes
+		;;
+	ask:*)
+		default_root=$(expr substr $default_root 5 100)
+		show_bootmenu=yes
+		;;
+esac
 
 # try to preselect item with same id as preset in default_root
 i=$MENU_MIN
@@ -557,7 +586,7 @@ if [ "$EVPOWER" != "" ]; then
 	(
 	$GETPOWERKEY > /dev/null
 	[ "$BRIGHTNESS" != "" ] && echo 0 > $BRIGHTNESS
-	${T2S} -c -B black
+	${T2S} -c -B 0
 	if [ "$PRODUCT" = "RX-51" ]; then
 		rm -f /etc/mtab
 		/etc/init.d/minishutdown
@@ -594,7 +623,7 @@ eval i=\$MENU_${MENU_IDX}_ID; [ -z "$i" ] || default_root=$i
 case "$default_root" in
 	off)
 		[ "$BRIGHTNESS" != "" ] && echo 0 > $BRIGHTNESS
-		${T2S} -c -B black
+		${T2S} -c -B 0
 		if [ "$PRODUCT" = "RX-51" ]; then
 			rm -f /etc/mtab
 			/etc/init.d/minishutdown
